#!/usr/bin/env python

import argparse
import functools
import time
from queue import Queue
from threading import Event, Thread
from typing import Callable

from colorama import Fore as Clr
from prompt_toolkit import PromptSession
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.patch_stdout import patch_stdout

from rtt_console.jlink_dongle import JLinkDongle, JLinkDongleException
from rtt_console.version import VERSION

DESCRIPTION = f'RTT Console {Clr.GREEN}v{VERSION}{Clr.RESET}'
CHIP_NAME_DEFAULT = 'STM32F407VE'

cmd_queue = Queue()


def exception_handling(func: Callable):

    @functools.wraps(func)
    def wrap(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except JLinkDongleException as e:
            print(e)
            return False

    return wrap


@exception_handling
def connect(jlink: JLinkDongle) -> bool:
    jlink.connect()
    return True


@exception_handling
def reconnect(jlink: JLinkDongle) -> bool:
    jlink.reconnect()
    return True


@exception_handling
def write_cmd(jlink: JLinkDongle, cmd: str) -> bool:
    jlink.write_rtt_sring(cmd)
    return True


@exception_handling
def read_data(jlink: JLinkDongle) -> str | bool:
    return jlink.read_rtt_string(0)


HELP_CMD = {'help', 'h'}
RECONNECT_CMD = {'reconnect', 'r'}

def conole_read_input(kill_evt: Event):
    session = PromptSession()
    input_cmd_string = ""
    while True:
        with patch_stdout(raw=True):
            try:
                input_cmd_string = session.prompt("> ", auto_suggest=AutoSuggestFromHistory())
            except KeyboardInterrupt as e:
                print(e)
                kill_evt.set()
                return
            cmd_queue.put(input_cmd_string)


def main():
    parser = argparse.ArgumentParser(prog='console', description=DESCRIPTION)
    parser.add_argument(f'-t',
                        '--target',
                        type=str,
                        help=f'Target chip name (example: {CHIP_NAME_DEFAULT})',
                        required=True)

    parser.add_argument(f'-s', '--speed', type=int, help='Target speed (default: auto)', required=False, default=0)
    args = parser.parse_args()

    try_to_reconnect = False
    if args.speed == 0:
        args.speed = 'auto'
    jlink = JLinkDongle(chip_name=args.target, speed=args.speed)
    if connect(jlink) == False:
        return
    kill_evt = Event()
    input: Thread = Thread(target=conole_read_input, args=([kill_evt]), daemon=True)
    input.start()
    while not kill_evt.wait(0.01):
        if try_to_reconnect:
            try_to_reconnect = False if reconnect(jlink) else True
            time.sleep(1)
            continue
        if not cmd_queue.empty():
            cmd = cmd_queue.get()
            if cmd in RECONNECT_CMD:
                try_to_reconnect = True
            else:
                if not write_cmd(jlink, cmd_queue.get()):
                    try_to_reconnect = True
        rx_data = read_data(jlink)
        if rx_data:
            print(rx_data, end="")
        if rx_data == False:
            try_to_reconnect = True


if __name__ == "__main__":
    main()



import functools
from dataclasses import dataclass, field

from colorama import Fore as Clr
from pylink import JLink, JLinkException, JLinkInterfaces

CHIP_NAME_DEFAULT = 'STM32F407VE'

class JLinkDongleException(Exception):

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)

@dataclass(slots=True)
class JLinkDongle:
    interface:JLinkInterfaces = JLinkInterfaces.SWD  # type: ignore
    speed:str|int = 'auto'
    chip_name:str = CHIP_NAME_DEFAULT
    jlink:JLink = field(init=False)

    @staticmethod
    def check_exception(func):
        @functools.wraps(func)
        def wrap(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except JLinkException as e:
                if func.__name__ in {self.read_rtt.__name__, self.write_rtt.__name__}:
                    raise JLinkDongleException(f"Do not read/write from RTT Terminal")
                raise JLinkDongleException(
                    f"{Clr.RED}ERROR:{Clr.RESET} method name: {Clr.YELLOW}{func.__name__}{Clr.RESET} : {e}")
        return wrap

    @check_exception
    def connect(self):
        self.jlink = JLink()
        self.jlink.disable_dialog_boxes()
        self.jlink.open()
        self.jlink.rtt_stop()
        self.jlink.set_tif(JLinkInterfaces.SWD)
        self.jlink.connect(chip_name=self.chip_name, speed=self.speed, verbose=True) # type: ignore
        self.jlink.rtt_start()
        endian = int.from_bytes(self.jlink._device.EndianMode, 'big') # type: ignore
        endian = {0: "Little", 1: "Big"}.get(endian, f"Unknown ({endian})")
        print()
        print(f"Connected to: {self.chip_name}")
        print(f"RTT RX buffers at {self.jlink.speed} kHz")
        print(f"connected to {endian}-Endian {self.jlink.core_name()}")
        print(f"running at {self.jlink.cpu_speed() / 1e6:.3f} MHz")

    @check_exception
    def read_rtt(self, terminal_number:int = 0) -> list:
        return self.jlink.rtt_read(terminal_number, self.jlink.MAX_BUF_SIZE)

    @check_exception
    def write_rtt(self, data:bytes, terminal_number:int = 0) -> None:
        self.jlink.rtt_write(terminal_number, data)

    def read_rtt_string(self, terminal_number:int = 0) -> str:
        data = self.read_rtt(terminal_number=terminal_number)
        if data:
            return bytes(data).decode('utf-8')
        return ""

    def write_rtt_sring(self, data:str, terminal_number:int = 0) -> None:
        self.write_rtt(str.encode(data, 'utf-8'), terminal_number)

    @check_exception
    def reconnect(self):
        self.jlink.close()
        # self.jlink.rtt_stop()
        self.connect()
